#include "h5si.h"

#include <iostream>
#include <complex>
#include <hdf5.h>

#include <mpi.h>

using namespace std;
using namespace blitz;

#define Complex complex<double>
#define H5Complex "cdouble"

void init_array(Array<Complex,3> A) {
	for (int i=0; i<A.extent(0); i++)
		for (int j=0; j<A.extent(1); j++)
			for (int k=0; k<A.extent(2); k++)
				A(i,j,k) = Complex(i*A.extent(1)*A.extent(2)*2 + j*A.extent(2)*2 + k*2, i*A.extent(1)*A.extent(2)*2 + j*A.extent(2)*2 + k*2+1);

}

void write_test() {
	Array<Complex,3> A(4,4,4);

	init_array(A);

	h5::File f;
	f.mpioInit(MPI_COMM_WORLD);
	f.open("foo.h5", "w");

	//Method1
	h5::Dataset ds1 = f.create_dataset("U.V1", h5::shape(4,4,4), H5Complex);
	ds1 << A.data();


	//Method 2
	h5::Plan plan;
	plan.set_plan(MPI_COMM_WORLD, h5::shape(4,4,4), h5::Select::all(3), h5::shape(4,4,4), h5::Select::all(3), h5::Dtype(H5Complex));
	h5::Dataset ds2 = f.create_dataset("U.V2", plan);
	ds2 << A.data();
}


void read_test() {
	Array<Complex,3> A(2,4,4);


	h5::File f("foo.h5", "r");
	f.mpioInit(MPI_COMM_WORLD);

	//Method 1
	f["U.V1"] >> A.data();
	cout << A << endl;


	//MEthod 2
	h5::Plan plan;
	plan.set_plan(MPI_COMM_WORLD, h5::shape(4,4,4), h5::Select::all(3), h5::shape(4,4,4), h5::Select::all(3), h5::Dtype(H5Complex));

	h5::Dataset ds = f["U.V2"];
	ds.set_plan(plan);
	ds >> A.data();
	cout << A << endl;



}

int main(int argc, char *argv[])
{
	MPI_Init(&argc, &argv);
	h5::init();

	write_test();
	// read_test();

	h5::finalize();
	MPI_Finalize();

	return 0;
}

