/* Tarang-2
 *
 * Copyright (C) 2008, 2009  Mahendra K. Verma
 *
 * Mahendra K. Verma
 * Indian Institute of Technology, Kanpur-208016
 * UP, India
 *
 * mkv@iitk.ac.in
 *
 * This file is part of Tarang-2 .
 *
 * Tarang-2 is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * Tarang-2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tarang-2; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, U
 */


/*! \file  IO.h
 * 
 *	@brief  Class declaration of input/output function.
 * 
 *
 *	@author  A. G. Chatterjee
 *	@version 2.0 MPI
 *	@date feb 2012
 *
 * @bug  No known bugs
 */

//*********************************************************************************************
#ifndef _H_H5DATASET
#define _H_H5DATASET

#include <hdf5.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <map>

#include <complex>

#include "h5plan.h"
#include "h5datatype.h"

namespace h5 {

	class Group;

	class Dataset
	{

	protected:
		hid_t id_;
		std::string name_;

		std::vector<hsize_t> shape_;

		hid_t filespace_dtype_;
		
		Group *parent_;

		std::string driver_;

	public:

#ifdef H5SI_ENABLE_MPI
		MPI_Comm MPI_COMMUNICATOR;
#endif
		Plan plan;

		Dataset();
		Dataset(const Dataset& ds);
		Dataset(Group *parent, std::string name);
		Dataset(Group *parent, std::string name, std::vector<hsize_t> shape, std::string data_type);
		Dataset(Group *parent, std::string name, Plan plan, std::string data_type="");

		~Dataset();

		void close();

		void create();

		void mpiioInit();

		hid_t id() const;
		hid_t dtype() const;
		std::string name() const;
		Group *parent() const;
		std::string driver() const;
		std::vector<hsize_t> shape() const;

		Dataset &set_plan(Plan plan);

		const Dataset &operator=(const Dataset &dataset);
	};

	const Dataset &operator>>(const Dataset &ds, void *data);
	const Dataset &operator<<(const Dataset &ds, const void *data);
	const Dataset &operator>>(const Dataset &ds, char *data);
	const Dataset &operator<<(const Dataset &ds, const char *data);
	const Dataset &operator>>(const Dataset &ds, signed char *data);
	const Dataset &operator<<(const Dataset &ds, const signed char *data);
	const Dataset &operator>>(const Dataset &ds, unsigned char *data);
	const Dataset &operator<<(const Dataset &ds, const unsigned char *data);
	const Dataset &operator>>(const Dataset &ds, short *data);
	const Dataset &operator<<(const Dataset &ds, const short *data);
	const Dataset &operator>>(const Dataset &ds, unsigned short *data);
	const Dataset &operator<<(const Dataset &ds, const unsigned short *data);
	const Dataset &operator>>(const Dataset &ds, int *data);
	const Dataset &operator<<(const Dataset &ds, const int *data);
	const Dataset &operator>>(const Dataset &ds, unsigned int *data);
	const Dataset &operator<<(const Dataset &ds, const unsigned int *data);
	const Dataset &operator>>(const Dataset &ds, long *data);
	const Dataset &operator<<(const Dataset &ds, const long *data);
	const Dataset &operator>>(const Dataset &ds, unsigned long *data);
	const Dataset &operator<<(const Dataset &ds, const unsigned long *data);
	const Dataset &operator>>(const Dataset &ds, long long *data);
	const Dataset &operator<<(const Dataset &ds, const long long *data);
	const Dataset &operator>>(const Dataset &ds, unsigned long long *data);
	const Dataset &operator<<(const Dataset &ds, const unsigned long long *data);
	const Dataset &operator>>(const Dataset &ds, float *data);
	const Dataset &operator<<(const Dataset &ds, const float *data);
	const Dataset &operator>>(const Dataset &ds, std::complex<float> *data);
	const Dataset &operator<<(const Dataset &ds, const std::complex<float> *data);
	const Dataset &operator>>(const Dataset &ds, double *data);
	const Dataset &operator<<(const Dataset &ds, const double *data);
	const Dataset &operator>>(const Dataset &ds, std::complex<double> *data);
	const Dataset &operator<<(const Dataset &ds, const std::complex<double> *data);
	const Dataset &operator>>(const Dataset &ds, long double *data);
	const Dataset &operator<<(const Dataset &ds, const long double *data);

}


#endif
//========================= Class declaration of BasicIO ends ============================== 

