#include "h5datatype.h"

#include <iostream>

namespace h5 {

	std::map<std::string, hid_t> Dtype::dtype_;
	hid_t Dtype::compound_type_native_complex_float_;
	hid_t Dtype::compound_type_native_complex_double_;

	Dtype::Dtype(std::string dtype_str) {
		this->selected_dtype_str_ = dtype_str;
	}

	void Dtype::finalize() {
		H5Tclose(compound_type_native_complex_float_);
		H5Tclose(compound_type_native_complex_double_);
	}

	void Dtype::init() {
		compound_type_native_complex_float_ = H5Tcreate(H5T_COMPOUND, 2*H5Tget_size(H5T_NATIVE_FLOAT));
		H5Tinsert(compound_type_native_complex_float_, "real", 0, H5T_NATIVE_FLOAT);
		H5Tinsert(compound_type_native_complex_float_, "imag", H5Tget_size(H5T_NATIVE_FLOAT), H5T_NATIVE_FLOAT);
		// H5Tlock(compound_type_native_complex_float_);

		compound_type_native_complex_double_ = H5Tcreate(H5T_COMPOUND, 2*H5Tget_size(H5T_NATIVE_DOUBLE));
		H5Tinsert(compound_type_native_complex_double_, "real", 0, H5T_NATIVE_DOUBLE);
		H5Tinsert(compound_type_native_complex_double_, "imag", H5Tget_size(H5T_NATIVE_DOUBLE), H5T_NATIVE_DOUBLE);
		// H5Tlock(compound_type_native_complex_double_);

		dtype_[">f32"] =  H5T_IEEE_F32BE;
		dtype_["<f32"] =  H5T_IEEE_F32LE;
		dtype_[">f64"] =  H5T_IEEE_F64BE;
		dtype_["<f64"] =  H5T_IEEE_F64LE;

		dtype_[">i8"] = H5T_STD_I8BE;
		dtype_["<i8"] = H5T_STD_I8LE;
		dtype_[">i16"] =  H5T_STD_I16BE;
		dtype_["<i16"] =  H5T_STD_I16LE;
		dtype_[">i32"] =  H5T_STD_I32BE;
		dtype_["<i32"] =  H5T_STD_I32LE;
		dtype_[">i64"] =  H5T_STD_I64BE;
		dtype_["<i64"] =  H5T_STD_I64LE;
		dtype_[">u8"] = H5T_STD_U8BE;
		dtype_["<u8"] = H5T_STD_U8LE;
		dtype_[">u16"] =  H5T_STD_U16BE;
		dtype_["<u16"] =  H5T_STD_U16LE;
		dtype_[">u32"] =  H5T_STD_U32BE;
		dtype_["<u32"] =  H5T_STD_U32LE;
		dtype_[">u64"] =  H5T_STD_U64BE;
		dtype_[">b8"] = H5T_STD_B8BE;
		dtype_["<b8"] = H5T_STD_B8LE;
		dtype_[">b16"] =  H5T_STD_B16BE;
		dtype_["<b16"] =  H5T_STD_B16LE;
		dtype_[">b32"] =  H5T_STD_B32BE;
		dtype_["<b32"] =  H5T_STD_B32LE;
		dtype_[">b64"] =  H5T_STD_B64BE;
		dtype_["<b64"] =  H5T_STD_B64LE;

		dtype_["S"] = H5T_C_S1;
		dtype_["a"] = H5T_C_S1;

		dtype_["char"] =  H5T_NATIVE_CHAR;
		dtype_["schar"] =  H5T_NATIVE_SCHAR;
		dtype_["uchar"] =  H5T_NATIVE_UCHAR;
		dtype_["short"] =  H5T_NATIVE_SHORT;
		dtype_["ushort"] =  H5T_NATIVE_USHORT;
		dtype_["int"] = H5T_NATIVE_INT;
		dtype_["uint"] =  H5T_NATIVE_UINT;
		dtype_["long"] =  H5T_NATIVE_LONG;
		dtype_["ulong"] =  H5T_NATIVE_ULONG;
		dtype_["llong"] =  H5T_NATIVE_LLONG;
		dtype_["ullong"] =  H5T_NATIVE_ULLONG;
		dtype_["float"] =  H5T_NATIVE_FLOAT;
		dtype_["double"] =  H5T_NATIVE_DOUBLE;
		dtype_["ldouble"] =  H5T_NATIVE_LDOUBLE;
		dtype_["b8"] = H5T_NATIVE_B8;
		dtype_["b16"] = H5T_NATIVE_B16;
		dtype_["b32"] = H5T_NATIVE_B32;
		dtype_["b64"] = H5T_NATIVE_B64;
		dtype_["opaque"] =  H5T_NATIVE_OPAQUE;
		dtype_["haddr"] =  H5T_NATIVE_HADDR;
		dtype_["hsize"] =  H5T_NATIVE_HSIZE;
		dtype_["hssize"] =  H5T_NATIVE_HSSIZE;
		dtype_["herr"] =  H5T_NATIVE_HERR;
		dtype_["hbool"] =  H5T_NATIVE_HBOOL;

		dtype_["cfloat"] =  compound_type_native_complex_float_;
		dtype_["cdouble"] =  compound_type_native_complex_double_;
	}
}
