/* Tarang-2
 *
 * Copyright (C) 2008, 2009  Mahendra K. Verma
 *
 * Mahendra K. Verma
 * Indian Institute of Technology, Kanpur-208016
 * UP, India
 *
 * mkv@iitk.ac.in
 *
 * This file is part of Tarang-2 .
 *
 * Tarang-2 is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * Tarang-2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tarang-2; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, U
 */


/*! \file  IO.h
 * 
 *	@brief  Class declaration of input/output function.
 * 
 *
 *	@author  A. G. Chatterjee
 *	@version 2.0 MPI
 *	@date feb 2012
 *
 * @bug  No known bugs
 */

//*********************************************************************************************
#ifndef _H_H5FILE
#define _H_H5FILE

#include <hdf5.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include "h5group.h"

namespace h5 {

	/**
	 *  This class is used to open an HDF5 file. A more elaborate class description.
	 */

	class File : public Group
	{
		hid_t fapl_; ///< Create a file access template to be used by corresponding drivers to set properties

		std::string filename_;
		std::string access_mode_;

	public:
		File();
		File(std::string filename, std::string access_mode);
		~File();

		int open(std::string filename, std::string access_mode);
		void close();
		void flush();

		void resetFapl();

		hid_t getFapl();
		void  setFapl(hid_t fapl);


		// driver specific initializers
		herr_t coreInit(size_t increment, hbool_t backing_store);
		herr_t stdioInit();

#ifdef H5SI_ENABLE_MPI

		herr_t mpioInit(MPI_Comm MPI_COMMUNICATOR);

#endif

		Node operator[](const std::string name);
		Node operator[](const hsize_t n);

		// void operator=(const File &file);

	};

}

#endif
//========================= Class declaration of BasicIO ends ============================== 

