/* Tarang-2
 *
 * Copyright (C) 2008, 2009  Mahendra K. Verma
 *
 * Mahendra K. Verma
 * Indian Institute of Technology, Kanpur-208016
 * UP, India
 *
 * mkv@iitk.ac.in
 *
 * This file is part of Tarang-2 .
 *
 * Tarang-2 is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * Tarang-2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tarang-2; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, U
 */


/*! \file  IO.h
 * 
 *	@brief  Class declaration of input/output function.
 * 
 *
 *	@author  A. G. Chatterjee
 *	@version 2.0 MPI
 *	@date feb 2012
 *
 * @bug  No known bugs
 */
#include <hdf5.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <string>
#include <vector>
#include <iostream>

#include "h5node.h"
#include "h5dataset.h"

//*********************************************************************************************
#ifndef _H_H5GROUP
#define _H_H5GROUP


namespace h5 {

	class Group
	{

	protected:
		bool is_root_;

		hid_t id_;
		std::string name_;

		hsize_t num_obj_;

		Group *parent_;

		std::string driver_;

//		std::vector<Node> child_nodes_opened;		//List of opened child nodes

	public:

#ifdef H5SI_ENABLE_MPI
		MPI_Comm MPI_COMMUNICATOR;
#endif

		typedef hsize_t size_type;

		Group();
		Group(const Group& ds);
		Group(Group *parent, std::string name, std::string access_mode);
		~Group();

		void register_node(Node node);
		void close();
		void close_all();

		Group createGroup(std::string name);
		Group requireGroup(std::string name);

		Dataset create_dataset(std::string name, std::vector<hsize_t> shape, std::string dtype) {
			return Dataset(this, name, shape, dtype);
		}
		Dataset create_dataset(std::string name, Plan plan, std::string dtype="") {
			return Dataset(this, name, plan, dtype);
		}
		// Dataset requireDataset(std::string name);

		hid_t id() {return this->id_;};
		std::string name();
		Group *parent();
		std::string driver() const;

		hsize_t getNumObjects();

		std::vector<std::string> keys(H5_index_t indexing_order=H5_INDEX_NAME);
		std::vector<Node> values();

		bool contains(std::string name);

		Node operator[](const std::string name);
		Node operator[](const hsize_t n);

		void operator=(const Group &group);

	    static void operator delete(void* ptr, std::size_t sz);		//Delete this group
	};

}

#endif
//========================= Class declaration of BasicIO ends ============================== 

