
#ifndef _H_H5SELECT
#define _H_H5SELECT

#include <string>
#include <algorithm>
#include <sstream>
#include <blitz/array.h>

#include <vector>



namespace h5 {
	
class Expression;

class Select {
	char sign_;				  // '+' or '-'
	std::vector<blitz::Range> range_;    // range to select along X, Y, and Z respectively

	public:

	blitz::Range Parse_indices(std::string indices);

	Select(blitz::Range range0) {
		this->sign_ = '+';
		this->range_.reserve(1);
		this->range_.push_back(range0);
	}

	Select(blitz::Range range0, blitz::Range range1) {
		this->sign_ = '+';
		this->range_.reserve(2);
		this->range_.push_back(range0);
		this->range_.push_back(range1);
	}

	Select(blitz::Range range0, blitz::Range range1, blitz::Range range2) {
		this->sign_ = '+';
		this->range_.reserve(3);
		this->range_.push_back(range0);
		this->range_.push_back(range1);
		this->range_.push_back(range2);
	}

	Select(blitz::Range range0, blitz::Range range1, blitz::Range range2, blitz::Range range3) {
		this->sign_ = '+';
		this->range_.reserve(4);
		this->range_.push_back(range0);
		this->range_.push_back(range1);
		this->range_.push_back(range2);
		this->range_.push_back(range3);
	}

	Select(blitz::Range range0, blitz::Range range1, blitz::Range range2, blitz::Range range3, blitz::Range range4) {
		this->sign_ = '+';
		this->range_.reserve(5);
		this->range_.push_back(range0);
		this->range_.push_back(range1);
		this->range_.push_back(range2);
		this->range_.push_back(range3);
		this->range_.push_back(range4);
	}

	Select(std::string select);

	static Select all(int nD);

	std::vector<blitz::Range> get_range() { return range_; }
	char get_sign() { return sign_; }

	void Negate() {this->sign_ = (this->sign_ == '+' ? '-' : '+');}

	void Pretty_print(std::string &str, bool print_positive_sign=false) const;

	blitz::Range operator [](int i) const {return range_[i];}
    blitz::Range& operator [](int i) {return range_[i];}
};

}

std::ostream& operator<< (std::ostream& stream, const h5::Select& select);

h5::Expression operator+(const h5::Select  &s1, const h5::Select  &s2); 
h5::Expression operator+(const h5::Expression &e1, const h5::Select  &s2);
h5::Expression operator+(const h5::Select  &s1, const h5::Expression &e2); 

h5::Expression operator-(const h5::Select  &s1, const h5::Select  &s2); 
h5::Expression operator-(const h5::Expression &e1, const h5::Select  &s2); 
h5::Expression operator-(const h5::Select  &s1, const h5::Expression &e2); 

#endif