/* Tarang-2
 *
 * Copyright (C) 2008, 2009  Mahendra K. Verma
 *
 * Mahendra K. Verma
 * Indian Institute of Technology, Kanpur-208016
 * UP, India
 *
 * mkv@iitk.ac.in
 *
 * This file is part of Tarang-2 .
 *
 * Tarang-2 is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * Tarang-2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tarang-2; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, U
 */


/*! \file  IO.h
 * 
 *	@brief  Class declaration of input/output function.
 * 
 *
 *	@author  A. G. Chatterjee
 *	@version 2.0 MPI
 *	@date feb 2012
 *
 * @bug  No known bugs
 */

//*********************************************************************************************
#ifndef _H_VECTOR_OPS
#define _H_VECTOR_OPS

#include <vector>
#include <algorithm>
#include <functional>
#include <blitz/array.h>
// #include <iostream>

namespace VecOps {

	template<typename T, int nD>
	static std::vector<T> to_vector(blitz::TinyVector<T,nD> tvec) {
		std::vector<T> vec(nD);

		vec.assign(tvec.data(), tvec.data()+nD);

		return vec;
	}

	template<typename T>
	std::vector<T> assign(std::vector<T> &vec, int val) {
		vec.assign(vec.size(), val);
		
		return vec;
	}

	template<typename T>
	std::vector<T> range_assign(std::vector<T> vec, blitz::Range r, T val) {
		for (typename std::vector<T>::size_type i=r.first(0); i<r.last(vec.size())-1; i++)
			vec[i] = val;
		
		return vec;
	}

	template<typename T>
	std::vector<T> add(std::vector<T> vec, T val) {
		std::transform(vec.begin(), vec.end(), vec.begin(),
				std::bind1st(std::plus<T>(), val));

		return vec;
	}

	template<typename T>
	std::vector<T> add(std::vector<T> vec1, std::vector<T> vec2) {
		std::transform(vec1.begin(), vec1.end(), vec2.begin(), vec1.begin(),
				std::plus<T>());

		return vec1;
	}

	template<typename T>
	std::vector<T> subtract(std::vector<T> vec, T val) {
		std::transform(vec.begin(), vec.end(), vec.begin(),
				std::bind1st(std::minus<T>(), val));

		return vec;
	}

	template<typename T>
	std::vector<T> subtract(std::vector<T> vec1, std::vector<T> vec2) {
		std::transform(vec1.begin(), vec1.end(), vec2.begin(), vec1.begin(),
				std::minus<T>());

		return vec1;
	}

	template<typename T>
	std::vector<T> multiply(std::vector<T> vec, T val) {
		std::transform(vec.begin(), vec.end(), vec.begin(),
				std::bind1st(std::multiplies<T>(), val));

		return vec;
	}

	template<typename T>
	std::vector<T> multiply(std::vector<T> vec1, std::vector<T> vec2) {
		std::transform(vec1.begin(), vec1.end(), vec2.begin(), vec1.begin(),
				std::multiplies<T>());

		return vec1;
	}

	template<typename T>
	std::vector<T> divide(std::vector<T> vec, T val) {
		std::transform(vec.begin(), vec.end(), vec.begin(),
				std::bind1st(std::divides<T>(), val));

		return vec;
	}

	template<typename T>
	std::vector<T> divide(std::vector<T> vec1, std::vector<T> vec2) {
		std::transform(vec1.begin(), vec1.end(), vec2.begin(), vec1.begin(),
				std::divides<T>());

		return vec1;
	}
}

#endif
//========================= Class declaration of BasicIO ends ============================== 

